/* Copyright 2012 Michael Haar

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/
package com.dngames.mobilewebcam;

import java.lang.ref.WeakReference;
import java.util.Date;

import com.dngames.mobilewebcam.PhotoSettings.Mode;

import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.Bitmap;
import android.net.ConnectivityManager;
import android.net.wifi.WifiManager;
import android.os.IBinder;
import android.os.PowerManager;
import android.util.Log;
import android.widget.Toast;

public class BackgroundPhoto implements ITextUpdater
{
	WeakReference<Context> mContext = null;
	
	public void TakePhoto(Context context, SharedPreferences prefs, PhotoSettings.Mode mode)
	{
		boolean ignoreinactivity = false; 
		long sincelastalive = System.currentTimeMillis() - MobileWebCam.gLastMotionKeepAliveTime;
		long keepalivetime = PhotoSettings.getEditInt(context, prefs, "motion_keepalive_refresh", 3600);
		if(keepalivetime > 0 && sincelastalive >= keepalivetime * 1000)
		{
			MobileWebCam.gLastMotionKeepAliveTime = System.currentTimeMillis();
			MobileWebCam.LogI("Taking keep alive picture!");
			ignoreinactivity = true;
		}
		
		String startTime = prefs.getString("activity_starttime", "00:00");
		String endTime = prefs.getString("activity_endtime", "24:00");
		if(Preview.CheckInTime(new Date(), startTime, endTime, false))
		{
			if(mWakeLock == null || !mWakeLock.isHeld())
			{
				// get lock for one picture
				PowerManager pwrmgr = (PowerManager)context.getSystemService(Context.POWER_SERVICE);
				mWakeLock = pwrmgr.newWakeLock(PowerManager.PARTIAL_WAKE_LOCK, "MobileWebCam.PhotoAlarm");
			    mWakeLock.acquire();
			    
			    Log.v("MobileWebCam", "PhotoAlarmWakeLock aquired!");
			}
			
			if(mode == Mode.HIDDEN)
			{
				ConnectivityManager connmgr = (ConnectivityManager)context.getSystemService(Context.CONNECTIVITY_SERVICE);
				if(connmgr.getNetworkPreference() == ConnectivityManager.TYPE_WIFI)
				{
					WifiManager wmgr = (WifiManager)context.getSystemService(Context.WIFI_SERVICE);
					if(mWifiLock == null || !mWifiLock.isHeld())
					{
						mWifiLock = wmgr.createWifiLock(WifiManager.WIFI_MODE_FULL, "MobileWebCam.PhotoAlarm");
						mWifiLock.acquire();
						Log.v("MobileWebCam", "WifiLock aquired!");
					}		
				}
				
				new PhotoService(context, this).TakePicture();
			}
			else if(!MobileWebCam.gInSettings)
			{
				if(MobileWebCam.gIsRunning)
				{
					Intent i = new Intent(context, MobileWebCam.class);
					i.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
					i.putExtra("alarm", "photo");
					context.startActivity(i);
				}
				else if(mode == Mode.BACKGROUND)
				{
					Intent i = new Intent(context, TakeHiddenPicture.class);
					i.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_FROM_BACKGROUND);
					i.putExtra("alarm", "photo");
					context.startActivity(i);
				}
				else
				{
					JobFinished();
				}
			}
			else
			{
				JobFinished();
			}
		}
	}	

	@Override
	public void UpdateText()
	{
		if(Preview.gPreview != null)
			Preview.gPreview.UpdateText();
	}

	@Override
	public void Toast(String msg, int length)
	{
		Context c = mContext.get();
		if(c != null)
		{
			SharedPreferences prefs = mContext.get().getSharedPreferences(MobileWebCam.SHARED_PREFS_NAME, 0);
			if(!prefs.getBoolean("no_messages", false))
			{
				if(mContext.get() != null)
					Toast.makeText(mContext.get(), msg, length).show();
			}
		}
		Log.i("MobileWebCam", "BackgroundPhoto: " + msg);
	}
	
	@Override
	public void SetPreview(Bitmap image)
	{
		if(Preview.gPreview != null && MobileWebCam.gIsRunning)
			Preview.gPreview.SetPreview(image);
	}
	
	private static PowerManager.WakeLock mWakeLock = null;
	private static WifiManager.WifiLock mWifiLock = null;
	
	@Override
	public void JobFinished()
	{
		if(Preview.gPreview != null)
			Preview.gPreview.JobFinished();
		
		releaseWakeLocks();
	}

	public static void releaseWakeLocks()
	{
		// release lock for one picture
		if(mWifiLock != null)
		{
			if(mWifiLock.isHeld())
				mWifiLock.release();
			Log.v("MobileWebCam", "WifiLock released!");
			mWifiLock = null;
		}
		
		if(mWakeLock != null)
		{
			PowerManager.WakeLock tmp = mWakeLock;
			Log.v("MobileWebCam", "PhotoAlarmWakeLock released!");
			mWakeLock = null;

			if(tmp.isHeld())
				tmp.release();
		}
	}
}